﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ChangeMaterialFormat {
    class ChangeMaterialFormat {
        /// <summary>
        /// アプリケーションのメイン エントリ ポイントです。
        /// </summary>
        [STAThread]
        static void Main() {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            Application.Run(new Form1());
        }

        private Form1 form;

        public ChangeMaterialFormat(Form1 form) {
            this.form = form;
        }

        /**
         * ●変更処理実行
         */
        public void change(string[] files) {
            // 処理中なら無視
            if (form.isBusy) {
                return;
            }

            // 処理中フラグを立てる。
            form.isBusy = true;

            // Formのテキストを保持しておく
            string originalFormText = form.Text;

            // 実行ファイルのパスを取得する
            Assembly myAssembly = Assembly.GetEntryAssembly();
            string exeFilePath = myAssembly.Location;
            string exeDirPath = Path.GetDirectoryName(exeFilePath);
            // 変換後の保存するフォルダ
            string convertDirPath = exeDirPath + @"\convert\";

            // フォルダが存在しなければ作成
            if (!Directory.Exists(convertDirPath)) {
                Directory.CreateDirectory(convertDirPath);
            }

            // ビットマップ準備
            Bitmap bmp = new Bitmap(1, 1);
            Bitmap newBmp = new Bitmap(1, 1);

            // 圧縮用にoptipngの実行準備
            ProcessStartInfo optipng = new ProcessStartInfo();
            optipng.FileName = "optipng.exe";
            optipng.WindowStyle = ProcessWindowStyle.Hidden;
            optipng.CreateNoWindow = true; // コンソール非表示

            // 倍率取得
            int scale = (int) form.numericUpDown1.Value;

            int count = 0;

            // ファイル名でループ
            foreach (string filePath in files) {
                // ファイルを取得して変換
                bmp = new Bitmap(filePath);
                // 変換後の画像領域を確保
                newBmp = new Bitmap(bmp.Width * scale, bmp.Height * scale);
                Graphics g = Graphics.FromImage(newBmp);

                // キャラクター一人分の高さ（想定は32ピクセル）
                int lineHeight = bmp.Height / 8;

                // ツクール２０００規格をＭＶＭＺ用に変換していく。
                Rectangle srcRect;
                Rectangle desRect;

                // 上：１行目→４行目
                srcRect = new Rectangle(0, 0, bmp.Width, lineHeight);
                desRect = new Rectangle(0, lineHeight * 3, bmp.Width, lineHeight);
                g.DrawImage(bmp, desRect, srcRect, GraphicsUnit.Pixel);

                // 右：２行目→３行目
                srcRect = new Rectangle(0, lineHeight * 1, bmp.Width, lineHeight);
                desRect = new Rectangle(0, lineHeight * 2, bmp.Width, lineHeight);
                g.DrawImage(bmp, desRect, srcRect, GraphicsUnit.Pixel);

                // 下：３行目→１行目
                srcRect = new Rectangle(0, lineHeight * 2, bmp.Width, lineHeight);
                desRect = new Rectangle(0, 0, bmp.Width, lineHeight);
                g.DrawImage(bmp, desRect, srcRect, GraphicsUnit.Pixel);

                // 左：４行目→２行目
                srcRect = new Rectangle(0, lineHeight * 3, bmp.Width, lineHeight);
                desRect = new Rectangle(0, lineHeight * 1, bmp.Width, lineHeight);
                g.DrawImage(bmp, desRect, srcRect, GraphicsUnit.Pixel);

                // 下の段

                // 上：５行目→８行目
                srcRect = new Rectangle(0, lineHeight * 4, bmp.Width, lineHeight);
                desRect = new Rectangle(0, lineHeight * 7, bmp.Width, lineHeight);
                g.DrawImage(bmp, desRect, srcRect, GraphicsUnit.Pixel);

                // 右：６行目→７行目
                srcRect = new Rectangle(0, lineHeight * 5, bmp.Width, lineHeight);
                desRect = new Rectangle(0, lineHeight * 6, bmp.Width, lineHeight);
                g.DrawImage(bmp, desRect, srcRect, GraphicsUnit.Pixel);

                // 下：７行目→５行目
                srcRect = new Rectangle(0, lineHeight * 6, bmp.Width, lineHeight);
                desRect = new Rectangle(0, lineHeight * 4, bmp.Width, lineHeight);
                g.DrawImage(bmp, desRect, srcRect, GraphicsUnit.Pixel);

                // 左：８行目→６行目
                srcRect = new Rectangle(0, lineHeight * 7, bmp.Width, lineHeight);
                desRect = new Rectangle(0, lineHeight * 5, bmp.Width, lineHeight);
                g.DrawImage(bmp, desRect, srcRect, GraphicsUnit.Pixel);

                // さらに倍率で描画
                srcRect = new Rectangle(0, 0, bmp.Width, bmp.Height);
                // 倍率分、開始位置を調整
                // ※やらないと左上のドットがずれる。
                // ※切り捨てになるのは意図通り
                // 1 -> 0, 3 -> 1
                int x = scale / 2;
                int y = scale / 2;
                desRect = new Rectangle(x, y, newBmp.Width, newBmp.Height);
                g.InterpolationMode = InterpolationMode.NearestNeighbor;
                g.DrawImage(newBmp, desRect, srcRect, GraphicsUnit.Pixel);

                // 解放
                g.Dispose();

                // 元画像が8bit画像（パレット有）の場合
                if (bmp.PixelFormat == PixelFormat.Format8bppIndexed) {
                    // 背景色を透明化（パレット左上）
                    // ※ＲＧＢ値が同一の色も透明化するがやむなし……。
                    newBmp.MakeTransparent(bmp.Palette.Entries[0]);
                }

                // フォルダ名を抜き出し
                string fileName = Path.GetFileName(filePath);
                string convertFilePath = convertDirPath + fileName;

                // 保存
                newBmp.Save(convertFilePath, System.Drawing.Imaging.ImageFormat.Png);

                // 出力したPNGをoptipngで圧縮する
                // ※256色形式を保持するための処置
                // -o1で圧縮Level1を指定
                optipng.Arguments = "\"" + convertFilePath + "\" -o1";

                // 処理終了を待つ
                using (Process exe = Process.Start(optipng)) {
                    exe.WaitForExit();
                }

                count++;

                // 進捗状況をFormテキスト（タイトル部分）に表示
                form.Text = originalFormText
                    + " - 実行中！(" + count.ToString() + "/" + files.Length + ")";
            }

            // 解放
            newBmp.Dispose();
            bmp.Dispose();

            // 名前を戻す。
            form.Text = originalFormText;
            // 処理中解除
            form.isBusy = false;

            //メッセージボックスを表示する
            MessageBox.Show(count + "件の変換が完了しました！");
        }
    }
}
